#include <stdlib.h>

#include "file_node.h"
#include "file_object.h"
#include "file.h"

struct dir_entry {
  FileNode node;
};

int Dir_GetEntryNumber(FileNode dir)
{
  FileObject object = FileNode_GetObject(dir);
  int size;
  size = FileObject_DataGetSize(object);
  return size / sizeof(struct dir_entry);
}

int Dir_IsEmpty(FileNode dir)
{
  FileNode node = NULL;
  int i, number;

  number = Dir_GetEntryNumber(dir);

  for (i = 0; i < number; i++) {
    node = Dir_GetNode(dir, i);
    if (node)
      return 0;
  }

  return 1;
}

int Dir_ExtendEntry(FileNode dir)
{
  FileObject object = FileNode_GetObject(dir);
  int index;
  index = FileObject_DataExtend(object, sizeof(struct dir_entry));
  index /= sizeof(struct dir_entry);
  return index - 1;
}

int Dir_ReduceEntry(FileNode dir)
{
  FileObject object = FileNode_GetObject(dir);
  int index;
  index = FileObject_DataReduce(object, sizeof(struct dir_entry));
  index /= sizeof(struct dir_entry);
  return index;
}

FileNode Dir_GetNode(FileNode dir, int index)
{
  FileObject object = FileNode_GetObject(dir);
  struct dir_entry entry;
  int size, offset;

  size = sizeof(entry);
  offset = size * index;

  if (FileObject_DataRead(object, offset, size, &entry) < size) {
    entry.node = NULL;
  }

  return entry.node;
}

FileNode Dir_SetNode(FileNode dir, int index, FileNode node)
{
  FileObject object = FileNode_GetObject(dir);
  struct dir_entry entry;
  int size, offset;

  size = sizeof(entry);
  offset = size * index;

  if (FileObject_DataRead(object, offset, size, &entry) < size) {
    return NULL;
  }

  entry.node = node;

  if (FileObject_DataWrite(object, offset, size, &entry) < size) {
    return NULL;
  }

  return entry.node;
}

FileNode Dir_SearchNode(FileNode dir, const char *name)
{
  FileNode node = NULL;
  int i, number;

  number = Dir_GetEntryNumber(dir);

  for (i = 0; i < number; i++) {
    node = Dir_GetNode(dir, i);
    if (!node)
      continue;
    if (FileNode_NameCompare(node, name) == 0)
      break;
  }
  if (i == number)
    node = NULL;

  return node;
}

FileNode Dir_InsertNode(FileNode dir, FileNode node)
{
  int i, number;

  number = Dir_GetEntryNumber(dir);

  for (i = 0; i < number; i++) {
    if (!Dir_GetNode(dir, i))
      break;
  }

  if (i == number) {
    i = Dir_ExtendEntry(dir);
  }

  Dir_SetNode(dir, i, node);

  return node;
}

FileNode Dir_AddNode(FileNode dir, const char *name)
{
  FileNode node;
  node = FileNode_Create(FileNode_GetPool(dir), name);
  return Dir_InsertNode(dir, node);
}

FileNode Dir_ExtractNode(FileNode dir, FileNode node)
{
  int i, number;

  number = Dir_GetEntryNumber(dir);

  for (i = 0; i < number; i++) {
    if (Dir_GetNode(dir, i) == node) {
      Dir_SetNode(dir, i, NULL);
      break;
    }
  }
  if (i == number) {
    node = NULL;
  } else {
    for (i = Dir_GetEntryNumber(dir); i > 0; i--) {
      if (Dir_GetNode(dir, i - 1))
	break;
      Dir_ReduceEntry(dir);
    }
  }

  return node;
}

FileNode Dir_DelNode(FileNode dir, FileNode node)
{
  node = Dir_ExtractNode(dir, node);
  if (!node) return NULL;
  return FileNode_Destroy(node);
}
