#ifndef __FS_H_INCLUDED__
#define __FS_H_INCLUDED__

#include <fuse.h>

#include "block.h"
#include "file_node.h"

struct fs_info {
  char *image_filename;
  int imagefile;
  BlockPool pool;
  FileNode root;
};

int my_getattr(const char *, struct stat *);
int my_readlink(const char *, char *, size_t);
int my_getdir(const char *, fuse_dirh_t, fuse_dirfil_t);
int my_mknod(const char *, mode_t, dev_t);
int my_mkdir(const char *, mode_t);
int my_unlink(const char *);
int my_rmdir(const char *);
int my_symlink(const char *, const char *);
int my_rename(const char *, const char *);
int my_link(const char *, const char *);
int my_chmod(const char *, mode_t);
int my_chown(const char *, uid_t, gid_t);
int my_truncate(const char *, off_t);
int my_utime(const char *, struct utimbuf *);
int my_open(const char *, struct fuse_file_info *);
int my_read(const char *, char *, size_t, off_t,
	    struct fuse_file_info *);
int my_write(const char *, const char *, size_t, off_t,
	     struct fuse_file_info *);
int my_statfs(const char *, struct statvfs *);
int my_flush(const char *, struct fuse_file_info *);
int my_release(const char *, struct fuse_file_info *);
int my_fsync(const char *, int, struct fuse_file_info *);
int my_setxattr(const char *, const char *, const char *, size_t, int);
int my_getxattr(const char *, const char *, char *, size_t);
int my_listxattr(const char *, char *, size_t);
int my_removexattr(const char *, const char *);
int my_opendir(const char *, struct fuse_file_info *);
int my_readdir(const char *, void *, fuse_fill_dir_t, off_t,
	       struct fuse_file_info *);
int my_releasedir(const char *, struct fuse_file_info *);
int my_fsyncdir(const char *, int, struct fuse_file_info *);
void *my_init(struct fuse_conn_info *conn);
void my_destroy(void *);
int my_access(const char *, int);
int my_create(const char *, mode_t, struct fuse_file_info *);
int my_ftruncate(const char *, off_t, struct fuse_file_info *);
int my_fgetattr(const char *, struct stat *, struct fuse_file_info *);
int my_lock(const char *, struct fuse_file_info *, int cmd,
	    struct flock *);
int my_utimens(const char *, const struct timespec tv[2]);
int my_bmap(const char *, size_t blocksize, uint64_t *idx);

#endif
